import argparse
import logging
from sizer import convert_size, files_in_folder


def parse_args():
    parser = argparse.ArgumentParser(
                    prog = 'Sizer',
                    description = 'Recursively return all of the files in a given directory sorted by size')
    parser.add_argument('-p', '--path', help='Path to directory', required=True)
    parser.add_argument('-o', '--order', help='Order for sorting, defaults to Descending', default="Descending")
    parser.add_argument('--head', help='Option to specify how many entries you want to list, defaults to 10', default=10, type=int)
    parser.add_argument('-mr', "--machine_readable", help='Option to specify if the size of the files should be left uncoverted to human readable format, defaults to False', default=False, action='store_true')
    parser.add_argument( '-log',
                     '--loglevel',
                     default='warning',
                     help='Provide logging level. Example --loglevel debug, default=warning' )

    return parser.parse_args() 

if __name__ == '__main__':
    args = parse_args()  # <- command line arguments parsing
    logging.basicConfig( level=args.loglevel.upper() )
    logging.info( 'Logging now setup.' )
    file_sizes = files_in_folder(args.path, args.order)
    
    for size, filepath in file_sizes[:args.head]:
        if not args.machine_readable:
            size = convert_size(size)
        print(f"{size}: {filepath}")



from pathlib import Path
import os
import math
from tqdm import tqdm
import logging

def convert_size(size_bytes: int) -> str:
    '''
        Converts input of raw number of bytes to humab readable format
        Example: 
            Input: 1024 byte
            Output: 1 KB
        Source: https://stackoverflow.com/a/14822210/10316860
    '''
    if size_bytes == 0:
        return "0 B"
    size_name = ("B", "KB", "MB", "GB", "TB", "PB", "EB", "ZB", "YB")
    i = int(math.floor(math.log(size_bytes, 1024)))
    p = math.pow(1024, i)
    s = round(size_bytes / p, 2)
    return f"{s} {size_name[i]}"

def get_size(file: Path) -> int:
    try:
        return os.path.getsize(file)
    except FileNotFoundError as ex:
        logging.info("A file was removed while the script was running, probably a temporary file")
        logging.info(ex)

    return 0

def files_in_folder(path: str, order: str) -> list():
    files = list(Path(path).rglob('*'))
    try:
        file_sizes = [(get_size(file), file) for file in tqdm(files)]
    except FileNotFoundError as ex:
        logging.info("A file was removed while the script was running, probably a temporary file")
        logging.info(ex)

    if "Descending" == order:
        file_sizes.sort(reverse=True)
    else:
        file_sizes.sort()

    return file_sizes